@@ -6,9 +6,7 @@ from network import Network, DEFAULT_SERVERS, DEFAULT_PORTS, pick_random_server
 from interface import Connection, Interface
 from simple_config import SimpleConfig, get_config, set_config
 import bitcoin
-import account
 import transaction
 from transaction import Transaction
 from plugins import BasePlugin
 from commands import Commands, known_commands
-import wizard as wizard
