@@ -26,6 +26,7 @@
 
 import hashlib
 import base64
+import os
 import re
 import hmac
 
@@ -37,6 +38,7 @@ import aes
 
 ################################## transactions
 
+FEE_STEP = 10000
 RECOMMENDED_FEE = 50000
 COINBASE_MATURITY = 100
 COIN = 100000000
@@ -158,7 +160,7 @@ hmac_sha_512 = lambda x,y: hmac.new(x, y, hashlib.sha512).digest()
 
 def is_new_seed(x, prefix=version.SEED_PREFIX):
     import mnemonic
-    x = mnemonic.prepare_seed(x)
+    x = mnemonic.normalize_text(x)
     s = hmac_sha_512("Seed version", x.encode('utf8')).encode('hex')
     return s.startswith(prefix)
 
@@ -171,16 +173,25 @@ def is_old_seed(seed):
         uses_electrum_words = True
     except Exception:
         uses_electrum_words = False
-
     try:
         seed.decode('hex')
         is_hex = (len(seed) == 32 or len(seed) == 64)
     except Exception:
         is_hex = False
-
     return is_hex or (uses_electrum_words and (len(words) == 12 or len(words) == 24))
 
 
+def seed_type(x):
+    if is_old_seed(x):
+        return 'old'
+    elif is_new_seed(x):
+        return 'standard'
+    elif is_new_seed(x, version.SEED_PREFIX_2FA):
+        return '2fa'
+    return ''
+
+is_seed = lambda x: bool(seed_type(x))
+
 # pywallet openssl private key implementation
 
 def i2o_ECPublicKey(pubkey, compressed=False):
@@ -207,7 +218,11 @@ def i2o_ECPublicKey(pubkey, compressed=False):
 ############ functions from pywallet #####################
 
 def hash_160(public_key):
-    md = hashlib.new('ripemd160')
+    if 'ANDROID_DATA' in os.environ:
+        from Crypto.Hash import RIPEMD
+        md = RIPEMD.new()
+    else:
+        md = hashlib.new('ripemd')
     md.update(sha256(public_key))
     return md.digest()
 
@@ -404,9 +419,17 @@ def msg_magic(message):
     return "\x18Bitcoin Signed Message:\n" + encoded_varint + message
 
 
-def verify_message(address, signature, message):
+def verify_message(address, sig, message):
     try:
-        EC_KEY.verify_message(address, signature, message)
+        public_key, compressed = pubkey_from_signature(sig, message)
+        # check public key using the address
+        pubkey = point_to_ser(public_key.pubkey.point, compressed)
+        addr = public_key_to_bc_address(pubkey)
+        if address != addr:
+            raise Exception("Bad signature")
+        # check message
+        h = Hash(msg_magic(message))
+        public_key.verify_digest(sig[1:], h, sigdecode = ecdsa.util.sigdecode_string)
         return True
     except Exception as e:
         print_error("Verification error: {0}".format(e))
@@ -487,6 +510,22 @@ class MyVerifyingKey(ecdsa.VerifyingKey):
         return klass.from_public_point( Q, curve )
 
 
+def pubkey_from_signature(sig, message):
+    if len(sig) != 65:
+        raise Exception("Wrong encoding")
+    nV = ord(sig[0])
+    if nV < 27 or nV >= 35:
+        raise Exception("Bad encoding")
+    if nV >= 31:
+        compressed = True
+        nV -= 4
+    else:
+        compressed = False
+    recid = nV - 27
+    h = Hash(msg_magic(message))
+    return MyVerifyingKey.from_signature(sig[1:], recid, h, curve = SECP256k1), compressed
+
+
 class MySigningKey(ecdsa.SigningKey):
     """Enforce low S values in signatures"""
 
@@ -518,41 +557,27 @@ class EC_KEY(object):
         assert public_key.verify_digest(signature, msg_hash, sigdecode = ecdsa.util.sigdecode_string)
         return signature
 
-    def sign_message(self, message, compressed, address):
+    def sign_message(self, message, is_compressed):
         signature = self.sign(Hash(msg_magic(message)))
         for i in range(4):
-            sig = chr(27 + i + (4 if compressed else 0)) + signature
+            sig = chr(27 + i + (4 if is_compressed else 0)) + signature
             try:
-                self.verify_message(address, sig, message)
+                self.verify_message(sig, message)
                 return sig
             except Exception:
                 continue
         else:
             raise Exception("error: cannot sign message")
 
-    @classmethod
-    def verify_message(self, address, sig, message):
-        if len(sig) != 65:
-            raise Exception("Wrong encoding")
-        nV = ord(sig[0])
-        if nV < 27 or nV >= 35:
-            raise Exception("Bad encoding")
-        if nV >= 31:
-            compressed = True
-            nV -= 4
-        else:
-            compressed = False
-        recid = nV - 27
 
-        h = Hash(msg_magic(message))
-        public_key = MyVerifyingKey.from_signature(sig[1:], recid, h, curve = SECP256k1)
+    def verify_message(self, sig, message):
+        public_key, compressed = pubkey_from_signature(sig, message)
         # check public key
-        public_key.verify_digest(sig[1:], h, sigdecode = ecdsa.util.sigdecode_string)
-        pubkey = point_to_ser(public_key.pubkey.point, compressed)
-        # check that we get the original signing address
-        addr = public_key_to_bc_address(pubkey)
-        if address != addr:
+        if point_to_ser(public_key.pubkey.point, compressed) != point_to_ser(self.pubkey.point, compressed):
             raise Exception("Bad signature")
+        # check message
+        h = Hash(msg_magic(message))
+        public_key.verify_digest(sig[1:], h, sigdecode = ecdsa.util.sigdecode_string)
 
 
     # ECIES encryption/decryption methods; AES-128-CBC with PKCS7 is used as the cipher; hmac-sha256 is used as the mac
@@ -798,3 +823,10 @@ def bip32_private_key(sequence, k, chain):
     for i in sequence:
         k, chain = CKD_priv(k, chain, i)
     return SecretToASecret(k, True)
+
+
+def xkeys_from_seed(seed, passphrase, derivation):
+    from mnemonic import Mnemonic
+    xprv, xpub = bip32_root(Mnemonic.mnemonic_to_seed(seed, passphrase))
+    xprv, xpub = bip32_private_derivation(xprv, "m/", derivation)
+    return xprv, xpub
