@@ -35,6 +35,10 @@ from util import *
 from i18n import _
 from util import profiler, PrintError, DaemonThread, UserCancelled
 
+plugin_loaders = {}
+hook_names = set()
+hooks = {}
+
 
 class Plugins(DaemonThread):
 
@@ -62,22 +66,21 @@ class Plugins(DaemonThread):
             m = loader.find_module(name).load_module(name)
             d = m.__dict__
             gui_good = self.gui_name in d.get('available_for', [])
-            # We register wallet types even if the GUI isn't provided
-            # otherwise the user gets a misleading message like
-            # "Unknown wallet type: 2fa"
-            details = d.get('registers_wallet_type')
-            if details:
-                self.register_plugin_wallet(name, gui_good, details)
             if not gui_good:
                 continue
+            details = d.get('registers_wallet_type')
+            if details:
+                self.register_wallet_type(name, gui_good, details)
+            details = d.get('registers_keystore')
+            if details:
+                self.register_keystore(name, gui_good, details)
             self.descriptions[name] = d
             if not d.get('requires_wallet_type') and self.config.get('use_' + name):
                 try:
                     self.load_plugin(name)
                 except BaseException as e:
                     traceback.print_exc(file=sys.stdout)
-                    self.print_error("cannot initialize plugin %s:" % name,
-                                     str(e))
+                    self.print_error("cannot initialize plugin %s:" % name, str(e))
 
     def get(self, name):
         return self.plugins.get(name)
@@ -86,6 +89,8 @@ class Plugins(DaemonThread):
         return len(self.plugins)
 
     def load_plugin(self, name):
+        if name in self.plugins:
+            return
         full_name = 'electrum_plugins.' + name + '.' + self.gui_name
         loader = pkgutil.find_loader(full_name)
         if not loader:
@@ -134,33 +139,38 @@ class Plugins(DaemonThread):
         requires = d.get('requires_wallet_type', [])
         return not requires or w.wallet_type in requires
 
-    def hardware_wallets(self, action):
-        wallet_types, descs = [], []
+    def get_hardware_support(self):
+        out = []
         for name, (gui_good, details) in self.hw_wallets.items():
             if gui_good:
                 try:
-                    p = self.wallet_plugin_loader(name)
-                    if action == 'restore' or p.is_enabled():
-                        wallet_types.append(details[1])
-                        descs.append(details[2])
+                    p = self.get_plugin(name)
+                    if p.is_enabled():
+                        out.append([name, details[2], p])
                 except:
                     traceback.print_exc()
                     self.print_error("cannot load plugin for:", name)
-        return wallet_types, descs
+        return out
 
-    def register_plugin_wallet(self, name, gui_good, details):
-        from wallet import Wallet
-
-        def dynamic_constructor(storage):
-            return self.wallet_plugin_loader(name).wallet_class(storage)
+    def register_wallet_type(self, name, gui_good, wallet_type):
+        from wallet import register_wallet_type, register_constructor
+        self.print_error("registering wallet type", (wallet_type, name))
+        def loader():
+            plugin = self.get_plugin(name)
+            register_constructor(wallet_type, plugin.wallet_class)
+        register_wallet_type(wallet_type)
+        plugin_loaders[wallet_type] = loader
 
+    def register_keystore(self, name, gui_good, details):
+        from keystore import register_keystore
+        def dynamic_constructor(d):
+            return self.get_plugin(name).keystore_class(d)
         if details[0] == 'hardware':
             self.hw_wallets[name] = (gui_good, details)
-        self.print_error("registering wallet %s: %s" %(name, details))
-        Wallet.register_plugin_wallet(details[0], details[1],
-                                      dynamic_constructor)
+            self.print_error("registering hardware %s: %s" %(name, details))
+            register_keystore(details[1], dynamic_constructor)
 
-    def wallet_plugin_loader(self, name):
+    def get_plugin(self, name):
         if not name in self.plugins:
             self.load_plugin(name)
         return self.plugins[name]
@@ -169,12 +179,9 @@ class Plugins(DaemonThread):
         while self.is_running():
             time.sleep(0.1)
             self.run_jobs()
-        self.print_error("stopped")
+        self.on_stop()
 
 
-hook_names = set()
-hooks = {}
-
 def hook(func):
     hook_names.add(func.func_name)
     return func
@@ -254,7 +261,7 @@ class DeviceUnpairableError(Exception):
     pass
 
 Device = namedtuple("Device", "path interface_number id_ product_key")
-DeviceInfo = namedtuple("DeviceInfo", "device description initialized")
+DeviceInfo = namedtuple("DeviceInfo", "device label initialized")
 
 class DeviceMgr(ThreadJob, PrintError):
     '''Manages hardware clients.  A client communicates over a hardware
@@ -289,9 +296,9 @@ class DeviceMgr(ThreadJob, PrintError):
 
     def __init__(self, config):
         super(DeviceMgr, self).__init__()
-        # Keyed by wallet.  The value is the device id if the wallet
+        # Keyed by xpub.  The value is the device id 
         # has been paired, and None otherwise.
-        self.wallets = {}
+        self.xpub_ids = {}
         # A list of clients.  The key is the client, the value is
         # a (path, id_) pair.
         self.clients = {}
@@ -300,6 +307,7 @@ class DeviceMgr(ThreadJob, PrintError):
         self.recognised_hardware = set()
         # For synchronization
         self.lock = threading.RLock()
+        self.hid_lock = threading.RLock()
         self.config = config
 
     def thread_jobs(self):
@@ -331,38 +339,35 @@ class DeviceMgr(ThreadJob, PrintError):
                 self.clients[client] = (device.path, device.id_)
         return client
 
-    def wallet_id(self, wallet):
+    def xpub_id(self, xpub):
         with self.lock:
-            return self.wallets.get(wallet)
+            return self.xpub_ids.get(xpub)
 
-    def wallet_by_id(self, id_):
+    def xpub_by_id(self, id_):
         with self.lock:
-            for wallet, wallet_id in self.wallets.items():
-                if wallet_id == id_:
-                    return wallet
+            for xpub, xpub_id in self.xpub_ids.items():
+                if xpub_id == id_:
+                    return xpub
             return None
 
-    def unpair_wallet(self, wallet):
+    def unpair_xpub(self, xpub):
         with self.lock:
-            if not wallet in self.wallets:
+            if not xpub in self.xpub_ids:
                 return
-            wallet_id = self.wallets.pop(wallet)
-            client = self.client_lookup(wallet_id)
-            self.clients.pop(client, None)
-        wallet.unpaired()
+            _id = self.xpub_ids.pop(xpub)
+        client = self.client_lookup(_id)
+        self.clients.pop(client, None)
         if client:
             client.close()
 
     def unpair_id(self, id_):
-        with self.lock:
-            wallet = self.wallet_by_id(id_)
-        if wallet:
-            self.unpair_wallet(wallet)
+        xpub = self.xpub_by_id(id_)
+        if xpub:
+            self.unpair_xpub(xpub)
 
-    def pair_wallet(self, wallet, id_):
+    def pair_xpub(self, xpub, id_):
         with self.lock:
-            self.wallets[wallet] = id_
-        wallet.paired()
+            self.xpub_ids[xpub] = id_
 
     def client_lookup(self, id_):
         with self.lock:
@@ -371,55 +376,58 @@ class DeviceMgr(ThreadJob, PrintError):
                     return client
         return None
 
-    def client_by_id(self, id_, handler):
+    def client_by_id(self, id_):
         '''Returns a client for the device ID if one is registered.  If
         a device is wiped or in bootloader mode pairing is impossible;
         in such cases we communicate by device ID and not wallet.'''
-        self.scan_devices(handler)
+        self.scan_devices()
         return self.client_lookup(id_)
 
-    def client_for_wallet(self, plugin, wallet, force_pair):
-        assert wallet.handler
+    def client_for_keystore(self, plugin, handler, keystore, force_pair):
+        self.print_error("getting client for keystore")
+        handler.update_status(False)
+        devices = self.scan_devices()
+        xpub = keystore.xpub
+        derivation = keystore.get_derivation()
+        client = self.client_by_xpub(plugin, xpub, handler, devices)
+        if client is None and force_pair:
+            info = self.select_device(plugin, handler, keystore, devices)
+            client = self.force_pair_xpub(plugin, handler, info, xpub, derivation, devices)
+        if client:
+            handler.update_status(True)
+        self.print_error("end client for keystore")
+        return client
 
-        devices = self.scan_devices(wallet.handler)
-        wallet_id = self.wallet_id(wallet)
-
-        client = self.client_lookup(wallet_id)
+    def client_by_xpub(self, plugin, xpub, handler, devices):
+        _id = self.xpub_id(xpub)
+        client = self.client_lookup(_id)
         if client:
             # An unpaired client might have another wallet's handler
             # from a prior scan.  Replace to fix dialog parenting.
-            client.handler = wallet.handler
+            client.handler = handler
             return client
 
         for device in devices:
-            if device.id_ == wallet_id:
-                return self.create_client(device, wallet.handler, plugin)
+            if device.id_ == _id:
+                return self.create_client(device, handler, plugin)
 
-        if force_pair:
-            return self.force_pair_wallet(plugin, wallet, devices)
-
-        return None
-
-    def force_pair_wallet(self, plugin, wallet, devices):
-        first_address, derivation = wallet.first_address()
-        assert first_address
 
+    def force_pair_xpub(self, plugin, handler, info, xpub, derivation, devices):
         # The wallet has not been previously paired, so let the user
         # choose an unpaired device and compare its first address.
-        info = self.select_device(wallet, plugin, devices)
 
         client = self.client_lookup(info.device.id_)
         if client and client.is_pairable():
             # See comment above for same code
-            client.handler = wallet.handler
+            client.handler = handler
             # This will trigger a PIN/passphrase entry request
             try:
-                client_first_address = client.first_address(derivation)
+                client_xpub = client.get_xpub(derivation)
             except (UserCancelled, RuntimeError):
                  # Bad / cancelled PIN / passphrase
-                client_first_address = None
-            if client_first_address == first_address:
-                self.pair_wallet(wallet, info.device.id_)
+                client_xpub = None
+            if client_xpub == xpub:
+                self.pair_xpub(xpub, info.device.id_)
                 return client
 
         # The user input has wrong PIN or passphrase, or cancelled input,
@@ -435,10 +443,8 @@ class DeviceMgr(ThreadJob, PrintError):
         '''Returns a list of DeviceInfo objects: one for each connected,
         unpaired device accepted by the plugin.'''
         if devices is None:
-            devices = self.scan_devices(handler)
-        devices = [dev for dev in devices if not self.wallet_by_id(dev.id_)]
-
-        states = [_("wiped"), _("initialized")]
+            devices = self.scan_devices()
+        devices = [dev for dev in devices if not self.xpub_by_id(dev.id_)]
         infos = []
         for device in devices:
             if not device.product_key in plugin.DEVICE_IDS:
@@ -446,51 +452,61 @@ class DeviceMgr(ThreadJob, PrintError):
             client = self.create_client(device, handler, plugin)
             if not client:
                 continue
-            state = states[client.is_initialized()]
-            label = client.label() or _("An unnamed %s") % plugin.device
-            descr = "%s (%s)" % (label, state)
-            infos.append(DeviceInfo(device, descr, client.is_initialized()))
+            infos.append(DeviceInfo(device, client.label(), client.is_initialized()))
 
         return infos
 
-    def select_device(self, wallet, plugin, devices=None):
+    def select_device(self, plugin, handler, keystore, devices=None):
         '''Ask the user to select a device to use if there is more than one,
         and return the DeviceInfo for the device.'''
         while True:
-            infos = self.unpaired_device_infos(wallet.handler, plugin, devices)
+            infos = self.unpaired_device_infos(handler, plugin, devices)
             if infos:
                 break
             msg = _('Could not connect to your %s.  Verify the cable is '
                     'connected and that no other application is using it.\n\n'
                     'Try to connect again?') % plugin.device
-            if not wallet.handler.yes_no_question(msg):
+            if not handler.yes_no_question(msg):
                 raise UserCancelled()
             devices = None
-
         if len(infos) == 1:
             return infos[0]
+        # select device by label
+        for info in infos:
+            if info.label == keystore.label:
+                return info
         msg = _("Please select which %s device to use:") % plugin.device
-        descriptions = [info.description for info in infos]
-        return infos[wallet.handler.query_choice(msg, descriptions)]
+        descriptions = [info.label + ' (%s)'%(_("initialized") if info.initialized else _("wiped")) for info in infos]
+        c = handler.query_choice(msg, descriptions)
+        if c is None:
+            raise UserCancelled()
+        info = infos[c]
+        # save new label
+        keystore.set_label(info.label)
+        handler.win.wallet.save_keystore()
+        return info
 
-    def scan_devices(self, handler):
+    def scan_devices(self):
         # All currently supported hardware libraries use hid, so we
         # assume it here.  This can be easily abstracted if necessary.
         # Note this import must be local so those without hardware
         # wallet libraries are not affected.
         import hid
-
         self.print_error("scanning devices...")
-
+        with self.hid_lock:
+            hid_list = hid.enumerate(0, 0)
         # First see what's connected that we know about
         devices = []
-        for d in hid.enumerate(0, 0):
+        for d in hid_list:
             product_key = (d['vendor_id'], d['product_id'])
             if product_key in self.recognised_hardware:
                 # Older versions of hid don't provide interface_number
                 interface_number = d.get('interface_number', 0)
+                serial = d['serial_number']
+                if len(serial) == 0:
+                    serial = d['path']
                 devices.append(Device(d['path'], interface_number,
-                                      d['serial_number'], product_key))
+                                      serial, product_key))
 
         # Now find out what was disconnected
         pairs = [(dev.path, dev.id_) for dev in devices]
