@@ -49,12 +49,6 @@ expiration_values = [
 ]
 
 
-def clean_text(seed_e):
-    text = unicode(seed_e.toPlainText()).strip()
-    text = ' '.join(text.split())
-    return text
-
-
 class Timer(QThread):
     stopped = False
 
@@ -472,7 +466,7 @@ class MyTreeWidget(QTreeWidget):
         key = str(item.data(0, Qt.UserRole).toString())
         text = unicode(item.text(column))
         self.parent.wallet.set_label(key, text)
-        self.parent.history_list.update()
+        self.parent.history_list.update_labels()
         self.parent.update_completions()
 
     def update(self):
