@@ -1,9 +1,10 @@
-from ..trezor.qt_generic import qt_plugin_class
+from ..trezor.qt_generic import QtPlugin
 from trezor import TrezorPlugin
 
 
-class Plugin(qt_plugin_class(TrezorPlugin)):
-    icon_file = ":icons/trezor.png"
+class Plugin(TrezorPlugin, QtPlugin):
+    icon_unpaired = ":icons/trezor_unpaired.png"
+    icon_paired = ":icons/trezor.png"
 
     @classmethod
     def pin_matrix_widget_class(self):
